@@ -559,6 +559,8 @@ func (b *Build) executeScript(ctx context.Context, trace JobTrace, executor Exec
 		// with after_script and later stages to still complete.
 		trace.SetCancelFunc(cancel)
 
+		b.printPolicyOptions()
+
 		for _, s := range b.Steps {
 			// after_script has a separate BuildStage. See common.BuildStageAfterScript
 			if s.Name == StepNameAfterScript {
@@ -1025,8 +1027,7 @@ func logTerminationError(logger buildlogger.Logger, name string, err error) {
 		return
 	}
 
-	logger = logger.WithFields(logrus.Fields{"error": err})
-	logger.Errorln(fmt.Sprintf("Job trace termination %q failed", name))
+	logger.WithFields(logrus.Fields{"error": err}).Errorln(fmt.Sprintf("Job trace termination %q failed", name))
 }
 
 func (b *Build) setExecutorStageResolver(resolver func() ExecutorStage) {
@@ -1679,7 +1680,7 @@ func (b *Build) printRunningWithHeader(trace JobTrace) {
 			"  on %s %s, system ID: %s",
 			b.Runner.Name,
 			b.Runner.ShortDescription(),
-			b.Runner.SystemIDState.GetSystemID(),
+			b.Runner.SystemID,
 		))
 	}
 	if featureInfo := b.getFeatureFlagInfo(); featureInfo != "" {
@@ -1693,6 +1694,35 @@ func (b *Build) printSettingErrors() {
 	}
 }
 
+func (b *Build) printPolicyOptions() {
+	if !b.JobResponse.PolicyOptions.PolicyJob {
+		return
+	}
+
+	b.logger.Infoln(fmt.Sprintf(`Job triggered by policy "%s".`, b.JobResponse.PolicyOptions.Name))
+
+	// VariableOverrideAllowed is optional.
+	// If not set, YAML variables defined in the policy are enforced with the highest precedence.
+	if b.JobResponse.PolicyOptions.VariableOverrideAllowed == nil {
+		b.logger.Infoln("Variables defined in the policy take precedence over matching user-defined CI/CD variables for this job.")
+		return
+	}
+
+	var message = "User-defined CI/CD variables are "
+	if *b.JobResponse.PolicyOptions.VariableOverrideAllowed {
+		message += "allowed in this job"
+	} else {
+		message += "ignored in this job"
+	}
+	// VariableOverrideExceptions acts as an allowlist when VariableOverrideExceptions is false
+	// and a denylist when it's true.
+	if b.JobResponse.PolicyOptions.VariableOverrideExceptions != nil {
+		message += fmt.Sprintf(" (except for %s)", strings.Join(b.JobResponse.PolicyOptions.VariableOverrideExceptions, ", "))
+	}
+	message += " according to the pipeline execution policy."
+	b.logger.Infoln(message)
+}
+
 func (b *Build) IsLFSSmudgeDisabled() bool {
 	return b.Settings().GitLFSSkipSmudge
 }
