@@ -300,6 +300,8 @@ type JobInfo struct {
 
 	TimeInQueueSeconds                       float64 `json:"time_in_queue_seconds"`
 	ProjectJobsRunningOnInstanceRunnersCount string  `json:"project_jobs_running_on_instance_runners_count"`
+	QueueSize                                int64   `json:"queue_size"`
+	QueueDepth                               int64   `json:"queue_depth"`
 }
 
 type GitInfoRefType string
@@ -418,12 +420,15 @@ var SupportedExecutorOptions = map[string][]string{
 type (
 	ImageDockerOptions struct {
 		executorOptions
-		Platform string `json:"platform"`
-		User     string `json:"user"`
+		Platform string        `json:"platform"`
+		User     StringOrInt64 `json:"user"`
 	}
+
+	StringOrInt64 string
+
 	ImageKubernetesOptions struct {
 		executorOptions
-		User string `json:"user"`
+		User StringOrInt64 `json:"user"`
 	}
 	ImageExecutorOptions struct {
 		executorOptions
@@ -471,7 +476,7 @@ func (iko *ImageKubernetesOptions) GetUIDGID() (int64, int64, error) {
 		return 0, 0, nil
 	}
 
-	user, group, ok := strings.Cut(iko.User, ":")
+	user, group, ok := strings.Cut(string(iko.User), ":")
 
 	uid, err := strconv.ParseInt(user, 10, 64)
 	if err != nil {
@@ -489,6 +494,22 @@ func (iko *ImageKubernetesOptions) GetUIDGID() (int64, int64, error) {
 	return uid, gid, err
 }
 
+func (si *StringOrInt64) UnmarshalJSON(data []byte) error {
+	var s string
+	if err := json.Unmarshal(data, &s); err == nil {
+		*si = StringOrInt64(s)
+		return nil
+	}
+
+	var i int64
+	if err := json.Unmarshal(data, &i); err == nil {
+		*si = StringOrInt64(strconv.FormatInt(i, 10))
+		return nil
+	}
+
+	return fmt.Errorf("StringOrInt: input not string or integer")
+}
+
 func (ieo *ImageExecutorOptions) UnmarshalJSON(data []byte) error {
 	type imageExecutorOptions ImageExecutorOptions
 	inner := imageExecutorOptions{}
@@ -606,6 +627,13 @@ type Artifact struct {
 
 type Artifacts []Artifact
 
+type PolicyOptions struct {
+	PolicyJob                  bool     `json:"execution_policy_job"`
+	Name                       string   `json:"policy_name"`
+	VariableOverrideAllowed    *bool    `json:"policy_variables_override_allowed,omitempty"`
+	VariableOverrideExceptions []string `json:"policy_variables_override_exceptions,omitempty"`
+}
+
 type Cache struct {
 	Key          string            `json:"key"`
 	Untracked    bool              `json:"untracked"`
@@ -725,10 +753,13 @@ type JobResponse struct {
 	Secrets       Secrets        `json:"secrets,omitempty"`
 	Hooks         Hooks          `json:"hooks,omitempty"`
 	Run           string         `json:"run"`
+	PolicyOptions PolicyOptions  `json:"policy_options,omitempty"`
 
 	TLSCAChain  string `json:"-"`
 	TLSAuthCert string `json:"-"`
 	TLSAuthKey  string `json:"-"`
+
+	JobRequestCorrelationID string `json:"-"`
 }
 
 // ValidateStepsJobRequest does the following:
