@@ -48,7 +48,6 @@ const minDockerDaemonVersion = "1.41"
 var (
 	getDefaultWindowsImageOnce sync.Once
 	defaultWindowsImage        string
-	systemIDState              = common.NewSystemIDState()
 )
 
 var windowsDockerImageTagMappings = map[string]string{
@@ -58,7 +57,6 @@ var windowsDockerImageTagMappings = map[string]string{
 
 func TestMain(m *testing.M) {
 	prebuilt.PrebuiltImagesPaths = []string{"../../out/helper-images/"}
-	_ = systemIDState.EnsureSystemID()
 
 	os.Exit(m.Run())
 }
@@ -188,8 +186,6 @@ func getRunnerConfigForOS(t *testing.T) *common.RunnerConfig {
 		image = getDefaultWindowsImage(t)
 	}
 
-	require.NoError(t, systemIDState.EnsureSystemID())
-
 	return &common.RunnerConfig{
 		RunnerSettings: common.RunnerSettings{
 			Executor: executor,
@@ -203,7 +199,6 @@ func getRunnerConfigForOS(t *testing.T) *common.RunnerConfig {
 		RunnerCredentials: common.RunnerCredentials{
 			Token: fmt.Sprintf("%x", md5.Sum([]byte(t.Name()))),
 		},
-		SystemIDState: systemIDState,
 	}
 }
 
@@ -384,7 +379,6 @@ func TestDockerCommandNoRootImage(t *testing.T) {
 					PullPolicy: common.StringOrArray{common.PullPolicyIfNotPresent},
 				},
 			},
-			SystemIDState: systemIDState,
 		},
 	}
 
@@ -412,7 +406,6 @@ func TestDockerCommandEntrypointWithStderrOutput(t *testing.T) {
 					featureflags.DisableUmaskForDockerExecutor: true,
 				},
 			},
-			SystemIDState: systemIDState,
 		},
 	}
 
@@ -440,7 +433,6 @@ func TestDockerCommandOwnershipOverflow(t *testing.T) {
 					featureflags.DisableUmaskForDockerExecutor: true,
 				},
 			},
-			SystemIDState: systemIDState,
 		},
 	}
 
@@ -485,7 +477,6 @@ func TestDockerCommandWithAllowedImagesRun(t *testing.T) {
 					PullPolicy:      common.StringOrArray{common.PullPolicyIfNotPresent},
 				},
 			},
-			SystemIDState: systemIDState,
 		},
 	}
 
@@ -548,7 +539,6 @@ func TestDockerCommandDisableEntrypointOverwrite(t *testing.T) {
 							DisableEntrypointOverwrite: test.disabled,
 						},
 					},
-					SystemIDState: systemIDState,
 				},
 			}
 
@@ -682,7 +672,6 @@ func TestDockerCommandTwoServicesFromOneImage(t *testing.T) {
 					PullPolicy: common.StringOrArray{common.PullPolicyIfNotPresent},
 				},
 			},
-			SystemIDState: systemIDState,
 		},
 	}
 
@@ -737,7 +726,6 @@ func TestDockerCommandServiceNameEmpty(t *testing.T) {
 					PullPolicy: common.StringOrArray{common.PullPolicyIfNotPresent},
 				},
 			},
-			SystemIDState: systemIDState,
 		},
 	}
 
@@ -774,7 +762,6 @@ func TestDockerCommandOutput(t *testing.T) {
 					PullPolicy: common.StringOrArray{common.PullPolicyIfNotPresent},
 				},
 			},
-			SystemIDState: systemIDState,
 		},
 	}
 
@@ -849,7 +836,6 @@ func getTestDockerJob(t *testing.T, job common.JobResponse) *common.Build {
 					Volumes:    []string{"/certs"},
 				},
 			},
-			SystemIDState: systemIDState,
 		},
 	}
 
@@ -952,7 +938,6 @@ func TestCacheInContainer(t *testing.T) {
 				},
 				Cache: &common.CacheConfig{},
 			},
-			SystemIDState: systemIDState,
 		},
 	}
 
@@ -1042,7 +1027,6 @@ func TestDockerImageNameFromVariable(t *testing.T) {
 					AllowedServices: []string{common.TestAlpineImage},
 				},
 			},
-			SystemIDState: systemIDState,
 		},
 	}
 
@@ -1077,7 +1061,6 @@ func TestDockerServiceNameFromVariable(t *testing.T) {
 					AllowedServices: []string{common.TestAlpineImage},
 				},
 			},
-			SystemIDState: systemIDState,
 		},
 	}
 
@@ -1167,7 +1150,6 @@ func TestDockerServiceHealthcheck(t *testing.T) {
 							WaitForServicesTimeout: 15,
 						},
 					},
-					SystemIDState: systemIDState,
 				},
 			}
 
@@ -1221,7 +1203,6 @@ func TestDockerServiceHealthcheckOverflow(t *testing.T) {
 				Executor: "docker",
 				Docker:   &common.DockerConfig{},
 			},
-			SystemIDState: systemIDState,
 		},
 	}
 
@@ -1260,7 +1241,6 @@ func TestDockerHandlesAliasDuplicates(t *testing.T) {
 				Executor: "docker",
 				Docker:   &common.DockerConfig{},
 			},
-			SystemIDState: systemIDState,
 		},
 	}
 
@@ -1374,7 +1354,6 @@ func testDockerVersion(t *testing.T, version string) {
 					CPUS:        "0.1",
 				},
 			},
-			SystemIDState: systemIDState,
 		},
 	}
 
@@ -1443,7 +1422,6 @@ func TestDockerCommandWithGitSSLCAInfo(t *testing.T) {
 					PullPolicy: common.StringOrArray{common.PullPolicyIfNotPresent},
 				},
 			},
-			SystemIDState: systemIDState,
 		},
 	}
 
@@ -1478,7 +1456,6 @@ func TestDockerCommandWithHelperImageConfig(t *testing.T) {
 				// the proxy_exec subcommand.
 				ProxyExec: func() *bool { v := false; return &v }(),
 			},
-			SystemIDState: systemIDState,
 		},
 	}
 
@@ -1571,7 +1548,6 @@ func TestDockerCommandWithDoingPruneAndAfterScript(t *testing.T) {
 					},
 				},
 			},
-			SystemIDState: systemIDState,
 		},
 	}
 
@@ -1856,7 +1832,6 @@ func TestChownAndUmaskUsage(t *testing.T) {
 							PullPolicy: common.StringOrArray{common.PullPolicyIfNotPresent},
 						},
 					},
-					SystemIDState: systemIDState,
 				},
 			}
 
@@ -2004,7 +1979,6 @@ func TestDockerCommandConflictingPullPolicies(t *testing.T) {
 					Image: common.TestAlpineImage,
 				},
 			},
-			SystemIDState: systemIDState,
 		},
 	}
 
@@ -2479,7 +2453,6 @@ func Test_ServiceLabels(t *testing.T) {
 					},
 				},
 			},
-			SystemIDState: systemIDState,
 		},
 	}
 
@@ -2567,7 +2540,6 @@ func TestDockerCommandWithPlatform(t *testing.T) {
 				Executor: "docker",
 				Docker:   &common.DockerConfig{},
 			},
-			SystemIDState: systemIDState,
 		},
 	}
 
@@ -2611,7 +2583,6 @@ func TestDockerCommandWithUser(t *testing.T) {
 				Executor: "docker",
 				Docker:   &common.DockerConfig{},
 			},
-			SystemIDState: systemIDState,
 		},
 	}
 
