@@ -36,11 +36,8 @@ import (
 )
 
 func testServiceFromNamedImage(t *testing.T, description, imageName, serviceName string) {
-	c := new(docker.MockClient)
-	defer c.AssertExpectations(t)
-
-	p := new(pull.MockManager)
-	defer p.AssertExpectations(t)
+	c := docker.NewMockClient(t)
+	p := pull.NewMockManager(t)
 
 	servicePart := fmt.Sprintf("-%s-0", strings.ReplaceAll(serviceName, "/", "__"))
 	containerNameRegex, err := regexp.Compile("runner-abcdef12-project-0-concurrent-0-[^-]+" + servicePart)
@@ -50,7 +47,7 @@ func testServiceFromNamedImage(t *testing.T, description, imageName, serviceName
 	networkID := "network-id"
 
 	e := &executor{
-		client: c,
+		dockerConn: &dockerConnection{Client: c},
 		info: system.Info{
 			OSType:       helperimage.OSTypeLinux,
 			Architecture: "amd64",
@@ -140,7 +137,6 @@ func testDockerConfigurationWithServiceContainer(
 	cce containerConfigExpectations,
 ) {
 	c, e := prepareTestDockerConfiguration(t, dockerConfig, cce, "alpine:latest", "alpine:latest")
-	defer c.AssertExpectations(t)
 
 	c.On("ContainerStart", mock.Anything, "abc", mock.Anything).
 		Return(nil).Once()
@@ -311,7 +307,6 @@ func TestDockerWithNoDockerConfigAndWithServiceImagePullPolicyAlways(t *testing.
 	}
 
 	c, e := prepareTestDockerConfiguration(t, dockerConfig, cce, "alpine:latest", "alpine:latest")
-	defer c.AssertExpectations(t)
 
 	c.On("ContainerStart", mock.Anything, "abc", mock.Anything).
 		Return(nil).Once()
@@ -356,8 +351,6 @@ func TestDockerWithDockerConfigAlwaysAndIfNotPresentAndWithServiceImagePullPolic
 	c.On("ContainerStart", mock.Anything, "abc", mock.Anything).
 		Return(nil).Once()
 
-	defer c.AssertExpectations(t)
-
 	err := e.createVolumesManager()
 	require.NoError(t, err)
 
@@ -811,16 +804,15 @@ func TestAddServiceHealthCheck(t *testing.T) {
 
 	for name, test := range tests {
 		t.Run(name, func(t *testing.T) {
-			client := new(docker.MockClient)
+			client := docker.NewMockClient(t)
 
 			if test.dockerClientAssertions != nil {
 				test.dockerClientAssertions(client)
 			}
-			defer client.AssertExpectations(t)
 
 			executor := &executor{
 				networkMode: container.NetworkMode(test.networkMode),
-				client:      client,
+				dockerConn:  &dockerConnection{Client: client},
 			}
 
 			service := &types.Container{
@@ -865,11 +857,9 @@ func Test_Executor_captureContainerLogs(t *testing.T) {
 
 	for name, tt := range tests {
 		t.Run(name, func(t *testing.T) {
-			c := new(docker.MockClient)
-			defer c.AssertExpectations(t)
-
+			c := docker.NewMockClient(t)
 			e := &executor{}
-			e.client = c
+			e.dockerConn = &dockerConnection{Client: c}
 
 			buf, err := trace.New()
 			require.NoError(t, err)
@@ -937,11 +927,10 @@ func Test_Executor_captureContainersLogs(t *testing.T) {
 	lentry.Out = &logs
 
 	stop := errors.New("don't actually try to stream the container's logs")
-	c := new(docker.MockClient)
-	defer c.AssertExpectations(t)
+	c := docker.NewMockClient(t)
 
 	e := &executor{services: containers}
-	e.client = c
+	e.dockerConn = &dockerConnection{Client: c}
 	e.BuildLogger = buildlogger.New(&common.Trace{Writer: &logs}, logrus.NewEntry(lentry), buildlogger.Options{})
 	e.Build = &common.Build{}
 
