@@ -23,7 +23,7 @@ var createVolumesManager = func(e *executor) (volumes.Manager, error) {
 		config.PermissionSetter = setter
 	}
 
-	volumesManager := volumes.NewManager(&e.BuildLogger, e.volumeParser, e.client, config, e.labeler)
+	volumesManager := volumes.NewManager(&e.BuildLogger, e.volumeParser, e.dockerConn, config, e.labeler)
 
 	return volumesManager, nil
 }
