@@ -43,7 +43,7 @@ const (
 	registerRunnerResponseRunnerProjectsLimitHit
 )
 
-var systemIDState = NewSystemIDState()
+const testSystemID = "test-system-id"
 
 var brokenCredentials = RunnerCredentials{
 	URL: "broken",
@@ -51,7 +51,7 @@ var brokenCredentials = RunnerCredentials{
 
 var brokenConfig = RunnerConfig{
 	RunnerCredentials: brokenCredentials,
-	SystemIDState:     systemIDState,
+	SystemID:          testSystemID,
 }
 
 func TestClients(t *testing.T) {
@@ -93,6 +93,8 @@ func TestClients(t *testing.T) {
 }
 
 func testRegisterRunnerHandler(w http.ResponseWriter, r *http.Request, response registerRunnerResponse, t *testing.T) {
+	w.Header().Add(correlationIDHeader, "foobar")
+
 	if r.URL.Path != "/api/v4/runners" {
 		w.WriteHeader(http.StatusNotFound)
 		return
@@ -113,8 +115,8 @@ func testRegisterRunnerHandler(w http.ResponseWriter, r *http.Request, response
 	res := make(map[string]interface{})
 
 	token := req["token"].(string)
-	require.NotEmpty(t, r.Header.Get(PrivateToken), "private-token header is required")
-	require.Equal(t, token, r.Header.Get("private-token"), "token in header and body must match")
+	require.NotEmpty(t, r.Header.Get(RunnerToken), "runner-token header is required")
+	require.Equal(t, token, r.Header.Get("runner-token"), "token in header and body must match")
 
 	switch token {
 	case validToken:
@@ -326,6 +328,7 @@ func TestRegisterRunnerOnRunnerLimitHit(t *testing.T) {
 			assert.Nil(t, res)
 			require.Len(t, h.entries, 1)
 			assert.Equal(t, "Registering runner... failed", h.entries[0].Message)
+			assert.Equal(t, "foobar", h.entries[0].Data["correlation_id"])
 			assert.Contains(t, h.entries[0].Data["status"], tc.expectedMessage)
 		})
 	}
@@ -368,8 +371,8 @@ func testUnregisterRunnerHandler(w http.ResponseWriter, r *http.Request, t *test
 	assert.NoError(t, err)
 
 	token := req["token"].(string)
-	require.NotEmpty(t, r.Header.Get(PrivateToken), "private-token header is required")
-	require.Equal(t, token, r.Header.Get("private-token"), "token in header and body must match")
+	require.NotEmpty(t, r.Header.Get(RunnerToken), "runner-token header is required")
+	require.Equal(t, token, r.Header.Get("runner-token"), "token in header and body must match")
 
 	switch token {
 	case validGlrtToken, validToken:
@@ -400,8 +403,8 @@ func testUnregisterRunnerManagerHandler(w http.ResponseWriter, r *http.Request,
 	assert.NoError(t, err)
 
 	token := req["token"].(string)
-	require.NotEmpty(t, r.Header.Get(PrivateToken), "private-token header is required")
-	require.Equal(t, token, r.Header.Get("private-token"), "token in header and body must match")
+	require.NotEmpty(t, r.Header.Get(RunnerToken), "runner-token header is required")
+	require.Equal(t, token, r.Header.Get("runner-token"), "token in header and body must match")
 
 	switch token {
 	case validGlrtToken:
@@ -527,8 +530,8 @@ func testVerifyRunnerHandler(w http.ResponseWriter, r *http.Request, legacyServe
 	res := make(map[string]interface{})
 
 	token := req["token"].(string)
-	require.NotEmpty(t, r.Header.Get(PrivateToken), "private-token header is required")
-	require.Equal(t, token, r.Header.Get("private-token"), "token in header and body must match")
+	require.NotEmpty(t, r.Header.Get(RunnerToken), "runner-token header is required")
+	require.Equal(t, token, r.Header.Get("runner-token"), "token in header and body must match")
 
 	switch token {
 	case validToken:
@@ -1018,6 +1021,8 @@ func getRequestJobResponse() map[string]interface{} {
 }
 
 func testRequestJobHandler(t *testing.T, w http.ResponseWriter, r *http.Request, jobResponse map[string]any) {
+	w.Header().Add(correlationIDHeader, "foobar")
+
 	if r.URL.Path != "/api/v4/jobs/request" {
 		w.WriteHeader(http.StatusNotFound)
 		return
@@ -1036,8 +1041,8 @@ func testRequestJobHandler(t *testing.T, w http.ResponseWriter, r *http.Request,
 	assert.NoError(t, err)
 
 	token := req["token"].(string)
-	require.NotEmpty(t, r.Header.Get(PrivateToken), "private-token header is required")
-	require.Equal(t, token, r.Header.Get("private-token"), "token in header and body must match")
+	require.NotEmpty(t, r.Header.Get(RunnerToken), "runner-token header is required")
+	require.Equal(t, token, r.Header.Get("runner-token"), "token in header and body must match")
 
 	switch token {
 	case validToken:
@@ -1053,7 +1058,7 @@ func testRequestJobHandler(t *testing.T, w http.ResponseWriter, r *http.Request,
 		return
 	}
 
-	assert.Equal(t, systemIDState.GetSystemID(), req["system_id"])
+	assert.Equal(t, testSystemID, req["system_id"])
 
 	if r.Header.Get(Accept) != "application/json" {
 		w.WriteHeader(http.StatusBadRequest)
@@ -1081,14 +1086,12 @@ func TestRequestJobWithUnsupportedOptions(t *testing.T) {
 	}))
 	defer s.Close()
 
-	require.NoError(t, systemIDState.EnsureSystemID())
-
 	validToken := RunnerConfig{
 		RunnerCredentials: RunnerCredentials{
 			URL:   s.URL,
 			Token: validToken,
 		},
-		SystemIDState: systemIDState,
+		SystemID: testSystemID,
 	}
 
 	c := NewGitLabClient()
@@ -1110,14 +1113,12 @@ func TestRequestJob(t *testing.T) {
 	}))
 	defer s.Close()
 
-	require.NoError(t, systemIDState.EnsureSystemID())
-
 	validToken := RunnerConfig{
 		RunnerCredentials: RunnerCredentials{
 			URL:   s.URL,
 			Token: validToken,
 		},
-		SystemIDState: systemIDState,
+		SystemID: testSystemID,
 	}
 
 	noJobsToken := RunnerConfig{
@@ -1125,7 +1126,7 @@ func TestRequestJob(t *testing.T) {
 			URL:   s.URL,
 			Token: "no-jobs",
 		},
-		SystemIDState: systemIDState,
+		SystemID: testSystemID,
 	}
 
 	invalidToken := RunnerConfig{
@@ -1133,7 +1134,7 @@ func TestRequestJob(t *testing.T) {
 			URL:   s.URL,
 			Token: invalidToken,
 		},
-		SystemIDState: systemIDState,
+		SystemID: testSystemID,
 	}
 
 	c := NewGitLabClient()
@@ -1185,14 +1186,29 @@ func TestRequestJob(t *testing.T) {
 		{
 			Level:   logrus.InfoLevel,
 			Message: "Checking for jobs... received",
+			Data: logrus.Fields{
+				"correlation_id": "foobar",
+				"job":            int64(10),
+				"repo_url":       "https://gitlab.example.com/test/test-project.git",
+				"runner":         "valid",
+			},
 		},
 		{
 			Level:   logrus.ErrorLevel,
 			Message: "Checking for jobs... forbidden",
+			Data: logrus.Fields{
+				"correlation_id": "foobar",
+				"runner":         "invalid",
+				"status":         "403 Forbidden",
+			},
 		},
 		{
 			Level:   logrus.ErrorLevel,
 			Message: "Checking for jobs... client error",
+			Data: logrus.Fields{
+				"correlation_id": "",
+				"status":         "only http or https scheme supported",
+			},
 		},
 	}
 
@@ -1200,13 +1216,11 @@ func TestRequestJob(t *testing.T) {
 	for i, l := range expectedLogs {
 		assert.Equal(t, l.Level, h.entries[i].Level)
 		assert.Equal(t, l.Message, h.entries[i].Message)
+		assert.Equal(t, l.Data, h.entries[i].Data)
 	}
 }
 
 func TestRequestJobWithSystemID(t *testing.T) {
-	systemIDState := NewSystemIDState()
-	require.NoError(t, systemIDState.EnsureSystemID())
-
 	s := httptest.NewServer(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
 		body, err := io.ReadAll(r.Body)
 		assert.NoError(t, err)
@@ -1216,7 +1230,7 @@ func TestRequestJobWithSystemID(t *testing.T) {
 		assert.NoError(t, err)
 
 		assert.NotEmpty(t, req["system_id"])
-		assert.Equal(t, systemIDState.GetSystemID(), req["system_id"])
+		assert.Equal(t, testSystemID, req["system_id"])
 	}))
 	defer s.Close()
 
@@ -1225,7 +1239,7 @@ func TestRequestJobWithSystemID(t *testing.T) {
 			URL:   s.URL,
 			Token: validToken,
 		},
-		SystemIDState: systemIDState,
+		SystemID: testSystemID,
 	}
 
 	c := NewGitLabClient()
@@ -1235,9 +1249,6 @@ func TestRequestJobWithSystemID(t *testing.T) {
 }
 
 func TestRequestJobAfterTooManyRequests(t *testing.T) {
-	systemIDState := NewSystemIDState()
-	require.NoError(t, systemIDState.EnsureSystemID())
-
 	s := httptest.NewServer(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
 		w.Header().Set(retryAfterHeader, "60")
 		w.WriteHeader(http.StatusTooManyRequests)
@@ -1249,7 +1260,7 @@ func TestRequestJobAfterTooManyRequests(t *testing.T) {
 			URL:   s.URL,
 			Token: validToken,
 		},
-		SystemIDState: systemIDState,
+		SystemID: testSystemID,
 	}
 
 	c := NewGitLabClient()
@@ -1281,6 +1292,8 @@ func setStateForUpdateJobHandlerResponse(w http.ResponseWriter, req map[string]i
 }
 
 func testUpdateJobHandler(w http.ResponseWriter, r *http.Request, t *testing.T) {
+	w.Header().Add(correlationIDHeader, "foobar")
+
 	if r.Method != http.MethodPut {
 		w.WriteHeader(http.StatusNotAcceptable)
 		return
@@ -1310,8 +1323,8 @@ func testUpdateJobHandler(w http.ResponseWriter, r *http.Request, t *testing.T)
 	assert.NoError(t, err)
 
 	token := req["token"].(string)
-	require.NotEmpty(t, r.Header.Get(PrivateToken), "private-token header is required")
-	require.Equal(t, token, r.Header.Get("private-token"), "token in header and body must match")
+	require.NotEmpty(t, r.Header.Get(JobToken), "job-token header is required")
+	require.Equal(t, token, r.Header.Get("job-token"), "token in header and body must match")
 
 	assert.Equal(t, "token", token)
 
@@ -1389,15 +1402,13 @@ func TestUpdateJob(t *testing.T) {
 		RunnerCredentials: RunnerCredentials{
 			URL: s.URL,
 		},
-		SystemIDState: systemIDState,
+		SystemID: testSystemID,
 	}
 
 	jobCredentials := &JobCredentials{
 		Token: "token",
 	}
 
-	require.NoError(t, systemIDState.EnsureSystemID())
-
 	c := NewGitLabClient()
 
 	for tn, tc := range testCases {
@@ -1419,12 +1430,15 @@ func TestUpdateJob(t *testing.T) {
 			assert.Equal(t, tc.updateJobInfo.Output.Checksum, h.entries[0].Data["checksum"])
 			if tc.additionalLog != nil {
 				assert.Equal(t, tc.additionalLog.Message, h.entries[1].Message)
+				assert.Equal(t, "foobar", h.entries[1].Data["correlation_id"])
 			}
 		})
 	}
 }
 
 func testUpdateJobKeepAliveHandler(w http.ResponseWriter, r *http.Request, t *testing.T) {
+	w.Header().Add(correlationIDHeader, "foobar")
+
 	if r.Method != http.MethodPut {
 		w.WriteHeader(http.StatusNotAcceptable)
 		return
@@ -1467,15 +1481,13 @@ func TestUpdateJobAsKeepAlive(t *testing.T) {
 		RunnerCredentials: RunnerCredentials{
 			URL: s.URL,
 		},
-		SystemIDState: systemIDState,
+		SystemID: testSystemID,
 	}
 
 	jobCredentials := &JobCredentials{
 		Token: "token",
 	}
 
-	require.NoError(t, systemIDState.EnsureSystemID())
-
 	c := NewGitLabClient()
 
 	type testCase struct {
@@ -1496,6 +1508,15 @@ func TestUpdateJobAsKeepAlive(t *testing.T) {
 				{
 					Level:   logrus.InfoLevel,
 					Message: "Submitting job to coordinator...ok",
+					Data: logrus.Fields{
+						"bytesize":        0,
+						"checksum":        "",
+						"code":            200,
+						"correlation_id":  "foobar",
+						"job":             int64(10),
+						"job-status":      "",
+						"update-interval": time.Duration(0),
+					},
 				},
 			},
 		},
@@ -1510,6 +1531,16 @@ func TestUpdateJobAsKeepAlive(t *testing.T) {
 				{
 					Level:   logrus.WarnLevel,
 					Message: "Submitting job to coordinator... job failed",
+					Data: logrus.Fields{
+						"bytesize":        0,
+						"checksum":        "",
+						"code":            200,
+						"correlation_id":  "foobar",
+						"job":             int64(11),
+						"job-status":      "canceled",
+						"status":          "200 OK",
+						"update-interval": time.Duration(0),
+					},
 				},
 			},
 		},
@@ -1524,6 +1555,16 @@ func TestUpdateJobAsKeepAlive(t *testing.T) {
 				{
 					Level:   logrus.WarnLevel,
 					Message: "Submitting job to coordinator... job failed",
+					Data: logrus.Fields{
+						"bytesize":        0,
+						"checksum":        "",
+						"code":            200,
+						"correlation_id":  "foobar",
+						"job":             int64(12),
+						"job-status":      "failed",
+						"status":          "200 OK",
+						"update-interval": time.Duration(0),
+					},
 				},
 			},
 		},
@@ -1538,6 +1579,15 @@ func TestUpdateJobAsKeepAlive(t *testing.T) {
 				{
 					Level:   logrus.InfoLevel,
 					Message: "Submitting job to coordinator...ok",
+					Data: logrus.Fields{
+						"bytesize":        0,
+						"checksum":        "",
+						"code":            200,
+						"correlation_id":  "foobar",
+						"job":             int64(13),
+						"job-status":      "canceling",
+						"update-interval": time.Duration(0),
+					},
 				},
 			},
 		},
@@ -1554,6 +1604,10 @@ func TestUpdateJobAsKeepAlive(t *testing.T) {
 			for i, l := range tc.expectedLogs {
 				assert.Equal(t, l.Level, h.entries[i].Level)
 				assert.Equal(t, l.Message, h.entries[i].Message)
+
+				if l.Data != nil {
+					assert.Equal(t, l.Data, h.entries[i].Data)
+				}
 			}
 		})
 	}
@@ -1582,7 +1636,7 @@ func getPatchServer(
 			return
 		}
 
-		assert.Equal(t, patchToken, r.Header.Get("JOB-TOKEN"))
+		assert.Equal(t, patchToken, r.Header.Get(JobToken))
 
 		body, err := io.ReadAll(r.Body)
 		assert.NoError(t, err)
@@ -1605,11 +1659,9 @@ func getPatchServer(
 		RunnerCredentials: RunnerCredentials{
 			URL: server.URL,
 		},
-		SystemIDState: systemIDState,
+		SystemID: testSystemID,
 	}
 
-	require.NoError(t, config.SystemIDState.EnsureSystemID())
-
 	return server, NewGitLabClient(), config
 }
 
@@ -1664,6 +1716,7 @@ func TestPatchTrace(t *testing.T) {
 				assert.Equal(t, patchTraceContent[offset:limit+1], body)
 
 				w.Header().Add(remoteStateHeader, tt.remoteState)
+				w.Header().Add(correlationIDHeader, "foobar")
 				w.WriteHeader(http.StatusAccepted)
 			}
 
@@ -1697,6 +1750,7 @@ func TestPatchTrace(t *testing.T) {
 			require.Len(t, h.entries, 3)
 			for _, entry := range h.entries {
 				assert.Equal(t, entry.Message, "Appending trace to coordinator...ok")
+				assert.Equal(t, "foobar", entry.Data["correlation_id"])
 			}
 		})
 	}
@@ -2142,11 +2196,9 @@ func TestUpdateIntervalHeaderHandling(t *testing.T) {
 
 				config := RunnerConfig{
 					RunnerCredentials: RunnerCredentials{URL: server.URL},
-					SystemIDState:     systemIDState,
+					SystemID:          testSystemID,
 				}
 
-				require.NoError(t, systemIDState.EnsureSystemID())
-
 				h := newLogHook(logrus.InfoLevel, logrus.WarnLevel)
 				logrus.AddHook(&h)
 
@@ -2323,13 +2375,13 @@ func testArtifactsUploadHandler(w http.ResponseWriter, r *http.Request, t *testi
 		return
 	}
 
-	if r.Header.Get("JOB-TOKEN") == "redirect" {
+	if r.Header.Get(JobToken) == "redirect" {
 		w.Header().Set("Location", "new-location")
 		w.WriteHeader(http.StatusTemporaryRedirect)
 		return
 	}
 
-	if r.Header.Get("JOB-TOKEN") != "token" {
+	if r.Header.Get(JobToken) != "token" {
 		w.WriteHeader(http.StatusForbidden)
 		return
 	}
@@ -2665,7 +2717,7 @@ func testArtifactsDownloadHandler(w http.ResponseWriter, r *http.Request) {
 		return
 	}
 
-	token := r.Header.Get("JOB-TOKEN")
+	token := r.Header.Get(JobToken)
 	if token == "invalid-token" {
 		w.WriteHeader(http.StatusForbidden)
 		return
@@ -2871,8 +2923,7 @@ func TestRunnerVersion(t *testing.T) {
 }
 
 func TestRunnerVersionToGetExecutorAndShellFeaturesWithTheDefaultShell(t *testing.T) {
-	executorProvider := MockExecutorProvider{}
-	defer executorProvider.AssertExpectations(t)
+	executorProvider := NewMockExecutorProvider(t)
 	executorProvider.On("GetDefaultShell").Return("my-default-executor-shell").Twice()
 	executorProvider.On("CanCreate").Return(true).Once()
 	executorProvider.On("GetFeatures", mock.Anything).Return(nil).Run(func(args mock.Arguments) {
@@ -2883,16 +2934,15 @@ func TestRunnerVersionToGetExecutorAndShellFeaturesWithTheDefaultShell(t *testin
 		info := args[1].(*ConfigInfo)
 		info.Gpus = "all"
 	})
-	RegisterExecutorProvider("my-test-executor", &executorProvider)
+	RegisterExecutorProviderForTest(t, "my-test-executor", executorProvider)
 
-	shell := MockShell{}
-	defer shell.AssertExpectations(t)
+	shell := NewMockShell(t)
 	shell.On("GetName").Return("my-default-executor-shell")
 	shell.On("GetFeatures", mock.Anything).Return(nil).Run(func(args mock.Arguments) {
 		features := args[0].(*FeaturesInfo)
 		features.Variables = true
 	})
-	RegisterShell(&shell)
+	RegisterShell(shell)
 
 	c := NewGitLabClient()
 	info := c.getRunnerVersion(RunnerConfig{
