@@ -148,11 +148,26 @@ func (b *BashWriter) Command(command string, arguments ...string) {
 	b.CheckForErrors()
 }
 
-func (b *BashWriter) CommandWithStdin(stdin, command string, arguments ...string) {
-	producer := b.buildCommand(b.escape, "echo", stdin)
-	consumer := b.buildCommand(b.escape, command, arguments...)
+// SetupGitCredHelper is the bash implementation of setting up the runner's default cred helper, which pulls out the job
+// token from the environment.
+func (b *BashWriter) SetupGitCredHelper(confFile, section, user string) {
+	helperSection := b.escape(section + ".helper")
+	userSection := b.escape(section + ".username")
+	emptyArg := `""`
 
-	b.Line(fmt.Sprintf("%s | %s", producer, consumer))
+	b.Line(
+		fmt.Sprintf(
+			`git config -f %[1]s --replace-all %[2]s %[3]s && `+
+				`git config -f %[1]s --add %[2]s %[4]s && `+
+				`git config -f %[1]s %[5]s %[6]s`,
+			b.escape(confFile),
+			helperSection,
+			emptyArg,
+			b.escape(credHelperCommand),
+			userSection,
+			user,
+		),
+	)
 	b.CheckForErrors()
 }
 
@@ -246,6 +261,15 @@ func (b *BashWriter) IfCmdWithOutput(cmd string, arguments ...string) {
 	b.Indent()
 }
 
+func (b *BashWriter) IfGitVersionIsAtLeast(version string) {
+	b.Linef(`current_ver="$(git version|cut -d ' ' -f 3)"`)
+	b.Linef(`required_ver=%q`, version)
+	b.Line(`minimum_ver="$(printf '%s\n%s' "$required_ver" "$current_ver" | sort --version-sort | head -n1)"`)
+	b.Linef(`if [ "$minimum_ver" = "$required_ver" ]; then`)
+	b.Printf("Git version at least %q", version)
+	b.Indent()
+}
+
 func (b *BashWriter) Else() {
 	b.Unindent()
 	b.Line("else")
@@ -390,16 +414,6 @@ func (b *BashShell) GetName() string {
 	return b.Shell
 }
 
-// Setting up an immediately invoked function is a common pattern in git, to handle arguments which git passes to
-// helpers by just appending those to the script snippet.
-// For more docs on that, and custom git cred helpers in general, see:
-// https://git-scm.com/docs/gitcredentials#_custom_helpers
-const bashGitCredHelperScript = `f(){ if [ "$1" = "get" ] ; then echo "password=${CI_JOB_TOKEN}" ; fi ; } ; f`
-
-func (b *BashShell) GetGitCredHelperCommand(os string) string {
-	return bashGitCredHelperScript
-}
-
 func (b *BashShell) GetEntrypointCommand(info common.ShellScriptInfo, probeFile string) []string {
 	script := b.bashDetectScript(info.Type == common.LoginShell)
 	if probeFile != "" {
